import { ForbiddenException, Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { PassportStrategy } from '@nestjs/passport';
import { Strategy, ExtractJwt } from 'passport-jwt';
import { ModuloService } from '../../modulo/modulo.service';
import { OperadorService } from '../../operador/operador.service';
import { UsuarioService } from '../../usuario/usuario.service';
import { Modulo } from '../../modulo/entity/modulo.entity';
import { Operador } from '../../operador/entity/operador.entity';
import { Usuario } from '../../usuario/entity/usuario.entity';
import { JwtPayloadAdmin } from '../dto/jwt-payload-admin';
import { JwtPayloadOperador } from '../dto/jwt-payload-operador';
import { JwtPayloadUsuario } from '../dto/jwt-payload-usuario';

@Injectable()
export class JwtStrategyService extends PassportStrategy(Strategy) {
  constructor(
    private configService: ConfigService,
    private moduloService: ModuloService,
    private operadorService: OperadorService,
    private usuarioService: UsuarioService,
  ) {
    // Configuración de JWT
    super({
      jwtFromRequest: ExtractJwt.fromAuthHeaderAsBearerToken(),
      secretOrKey: configService.get<string>('AUTH_SECRETKEY'),
    });
  }

  // Función que se ejecuta después de recibir un token válido
  async validate(
    payload: JwtPayloadAdmin | JwtPayloadOperador | JwtPayloadUsuario, // Payload del JWT que se mandó del client
  ) {
    const user: { modulo?: Modulo; usuario?: Usuario; operador?: Operador } =
      {};

    // Si el payload contiene un atributo llamado Usuario
    if ('Usuario' in payload)
      // Buscar y guardar
      await this.usuarioService
        .findById(payload.Usuario.id_usuario)
        .then((usuario) => (user.usuario = usuario));
    // Si el payload contiene un atributo llamado Operador
    else if ('Operador' in payload)
      // Buscar y guardar
      await this.operadorService
        .findById(payload.Operador.id_operador)
        .then((operador) => {
          // Validar que la cuenta de este operador este activa
          if (!operador.activo)
            throw new ForbiddenException('Esta cuenta esta desactivada.');
          user.operador = operador;
          // Si es una cuenta de operador
          if (operador.tipoUsuario.id_tipo_usuario === 4) {
            // Si el payload contiene un atributo llamado Modulo
            if ('Modulo' in payload)
              // Buscar y guardar
              return this.moduloService
                .findFullInfoById(payload.Modulo.id_modulo)
                .then((modulo) => (user.modulo = modulo));
            else
              throw new ForbiddenException(
                'Credenciales no válidas, inicia sesión de nuevo.',
              );
          }
        });
    return user; // Este objeto es el que extraemos de req.user en el controller
  }
}
