import {
  Body,
  Controller,
  Get,
  Post,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { ModeloService } from './modelo.service';
import { ValidarUsuarioService } from '../validar-usuario/validar-usuario.service';
import { Operador } from '../operador/entity/operador.entity';
import { CreateModeloDto } from './dto/input/create.dto';
import { ModeloDto } from './dto/input/modelo.dto';
import { MessageOutputDto } from '../dto/output/message.dto';
import { ModeloOutputDto } from './dto/output/modelo.dto';

@Controller('modelo')
@ApiTags('modelo')
export class ModeloController {
  constructor(
    private modeloService: ModeloService,
    private validarUsuarioService: ValidarUsuarioService,
  ) {}

  @Serealize(MessageOutputDto)
  @Post()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Crear un nuevo modelo.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: { ejemplo: { value: { modelo: '', tipo: '' } } },
  })
  create(@Request() req, @Body() body: CreateModeloDto) {
    const superAdmin: Operador = req.user.operador;

    this.validarUsuarioService.validarSuperAdmin(superAdmin);
    return this.modeloService.create(body.modelo, body.tipo);
  }

  @Serealize(ModeloOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Todos los modelos dependiendo del tipo.' })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Tipo de marca que se busca.',
    name: 'tipo',
    type: 'text',
  })
  get(@Request() req, @Query() query: ModeloDto) {
    const operador: Operador = req.user.operador;

    this.validarUsuarioService.validarOperador(operador);
    return this.modeloService.findAll(query.tipo);
  }
}
