import { forwardRef, Module } from '@nestjs/common';
import { PassportModule } from '@nestjs/passport';
import { TypeOrmModule } from '@nestjs/typeorm';
import { AppGateway } from '../app.gateway';
import { PrestamoController } from './prestamo.controller';
import { PrestamoService } from './prestamo.service';
import { Prestamo } from './entity/prestamo.entity';
import { FullInformacionPrestamoView } from './entity/views/full-informacion-prestamo.view';
import { InformacionPrestamoView } from './entity/views/informacion-prestamo.view';
import { PrestamoInformacionView } from './entity/views/prestamo-informacion.view';
import { EquipoModule } from '../equipo/equipo.module';
import { InstitucionModule } from '../institucion/institucion.module';
import { InstitucionDiaModule } from '../institucion-dia/institucion-dia.module';
import { InstitucionProgramaModule } from '../institucion-programa/institucion-programa.module';
import { InstitucionTipoCarritoModule } from '../institucion-tipo-carrito/institucion-tipo-carrito.module';
import { InstitucionTipoEntradaModule } from '../institucion-tipo-entrada/institucion-tipo-entrada.module';
import { InstitucionUsuarioModule } from '../institucion-usuario/institucion-usuario.module';
import { ModuloModule } from '../modulo/modulo.module';
import { MultaModule } from '../multa/multa.module';
import { OperadorModule } from '../operador/operador.module';
import { PasscodeModule } from '../passcode/passcode.module';
import { StatusModule } from '../status/status.module';
import { TipoUsuarioModule } from '../tipo-usuario/tipo-usuario.module';
import { UsuarioModule } from '../usuario/usuario.module';

@Module({
  imports: [
    EquipoModule,
    InstitucionModule,
    InstitucionDiaModule,
    InstitucionProgramaModule,
    InstitucionTipoCarritoModule,
    InstitucionTipoEntradaModule,
    InstitucionUsuarioModule,
    ModuloModule,
    forwardRef(() => MultaModule),
    OperadorModule,
    forwardRef(() => PasscodeModule),
    PassportModule.register({ defaultStrategy: 'jwt' }),
    StatusModule,
    TipoUsuarioModule,
    TypeOrmModule.forFeature([
      Prestamo,
      FullInformacionPrestamoView,
      InformacionPrestamoView,
      PrestamoInformacionView,
    ]),
    UsuarioModule,
  ],
  controllers: [PrestamoController],
  providers: [AppGateway, PrestamoService],
  exports: [PrestamoService],
})
export class PrestamoModule {}
